# coding: utf-8
from astroid import ImportFrom
from astroid import Module


def levels_to_toplevel(node: ImportFrom) -> int:
    """Returns the number of packages up to toplevel of project. """
    module: Module = node.root()
    dots_count = module.name.count('.')
    if module.file.endswith('__init__.py'):
        return dots_count + 1
    return dots_count


def is_relative(node: ImportFrom) -> bool:
    """Returns True, if import is relative, False otherwise. """
    return node.level is not None and node.level > 0



# coding: utf-8
from astroid import ImportFrom
from pylint.checkers import BaseChecker
from pylint.interfaces import IAstroidChecker

from pylint_relative_imports.utils import is_relative
from pylint_relative_imports.utils import levels_to_toplevel


class RelativeImportChecker(BaseChecker):
    __implements__ = IAstroidChecker

    name = 'relative-import-from-root'
    priority = -1
    msgs = {
        'C0001': (
            'Unacceptable relative import from another package',
            'relative-import-from-root',
            'Consider using absolute import',
        ),
    }

    def visit_importfrom(self, node: ImportFrom):
        if not is_relative(node):
            return
        module_levels_to_toplevel = levels_to_toplevel(node)
        if node.level >= module_levels_to_toplevel:
            self.add_message(
                'C0001',
                line=node.lineno,
                node=node,
                col_offset=node.col_offset,
            )


def register(linter):
    linter.register_checker(RelativeImportChecker(linter))



# coding: utf-8
from setuptools import setup


setup(
    name='pylint-relative-imports',
    version='0.0.4',
    package_dir={'': 'src'},
    packages=['pylint_relative_imports'],
    url='https://github.com/kubikvid/pylint-relative-imports',
    license='MIT License',
    author='Andrey Chernykh',
    author_email='moonquiz@ya.ru',
    description=(
        'Pylint plugin which adds linter error for '
        'relative imports from the root of the project'
    ),
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ),
    install_requires=(
        'pylint>=2.5.0,<3',
    )
)
